#include "openglbaumzeiger.h"

OpenGLBaumZeiger::OpenGLBaumZeiger(QWidget *parent) : QGLWidget(parent)
{
    objektRef=0;
    zDist=5.0;
    rotX=20.0;
    rotY=45.0;
    rekTiefe=6;
    laenge=0.25;
    verkuerzung=0.8;
    winkel=45.0;
    winkelSkal=1.0;
}

OpenGLBaumZeiger::~OpenGLBaumZeiger()
{
    makeCurrent();
    glDeleteLists(objektRef,1);
}

void OpenGLBaumZeiger::initializeGL()
{
    glShadeModel(GL_FLAT);
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    erstelleBaum();
}

void OpenGLBaumZeiger::resizeGL(int width,int height)
{
    int side = qMin(width, height);
    glViewport((width - side) / 2, (height - side) / 2, side, side);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1.0,1.0,-1.0,1.0,1.0,15.0);
    glMatrixMode(GL_MODELVIEW);
}

void OpenGLBaumZeiger::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT);
    glLoadIdentity();
    glTranslatef(0,0,-zDist);
    glRotatef(rotX,1,0,0);
    glRotatef(rotY,0,1,0);
    glCallList(objektRef);
}

void OpenGLBaumZeiger::mouseMoveEvent(QMouseEvent *e)
{
    QPoint bew=e->pos()-letzteMausPos;
    if(e->buttons() & Qt::LeftButton)
    {
        rotX+=360.0*float(bew.y()) / height();
        rotY+=360.0*float(bew.x()) / width();
    }
    letzteMausPos=e->pos();
    updateGL();
}

void OpenGLBaumZeiger::erstelleBaum()
{
    makeCurrent();
    if(objektRef != 0)
    {
        glDeleteLists(objektRef,1);
    }
    objektRef = glGenLists(1);
    glNewList(objektRef, GL_COMPILE);
    glColor3f(1.0,1.0,1.0);
    glPushMatrix();
    glTranslatef(0,-laenge,0);
    erstelleBaumRek(laenge,winkel,0);
    glPopMatrix();
    glEndList();
}

void OpenGLBaumZeiger::erstelleBaumRek(float laenge,float winkel,int rek)
{
    if(rek != rekTiefe)
    {
        glBegin(GL_LINES);
        glVertex2f(0.0,0.0); glVertex2f(0.0,laenge);
        glEnd();
        glTranslatef(0,laenge,0);
        glPushMatrix();
        for(int k=0;k<4;k++)
        {
            glPushMatrix();
            glRotatef(winkel,0,0,1);
            erstelleBaumRek(laenge*verkuerzung,winkel*winkelSkal,rek+1);
            glPopMatrix();
            glRotatef(90.0,0,1,0);
        }
        glPopMatrix();
        //glRotatef(-winkel,0,0,1);
        //erstelleBaumRek(laenge*verkuerzung,winkel*winkelSkal,rek+1);
        //glPopMatrix();

    }
}
